#include "RROScommands.bas"     // open this file like a normal program to see its contents
gosub InitCommands  // found in the above file
Real = false   //set to TRUE to control the real ARLO robot

main:
  gosub InitRobot
  Cur = 4    
  while not Lost
    Dest = 2
    gosub "From"+ToString(Cur)+"to"+ToString(Dest)
    Dest = 4
    gosub "From"+ToString(Cur)+"to"+ToString(Dest)
  wend
  xyString 500,300,"Robot is LOST"
end


InitRobot:
  if Real
    rCommPort 8   // change to YOUR Bluetooth Port number
    rLocate ARLO,0
    rCommand(SetCompassType, HMC5883)  // or HMC6352
    rCommand(SetRobotAngle,95)   // change to make your NORTH a particular direction
    rCommand(SetTurnStyle,30)
    WallDistance = 14
  else
    gosub DrawFloorPlan
    rLocate 580,350
    rSlip 2 // gives 2% error for movements and sensor readings
    WallDistance = 35
  endif
  OnLeft=1
  OnRight=2
  Lost = False
return

From4to2:
  if Lost then return
  call Face(WEST)
  call FindWall()
  call FollowWall(OnLeft)
  if Real  
    call Face(WEST+20)
  else
    call Face(WEST)
  endif
  if Real
    rForward 10 // Arlo measures from edge of robot
  else
    rForward 40 // simulator measures from center of robot
  endif 
  if Real 
    call MoveThroughDoor(SOUTH+15)
  else
    call MoveThroughDoor(SOUTH)
  endif
  call Face(135)
  delay 1000
  Cur = 2
return 

From2to4:  
  if Lost then return
  call MoveThroughDoor(NORTH+10)
  call Face(EAST)
  call FindWall()
  call FollowWall(OnRight)
  call Face(EAST)
  if Real
    rForward 5
  else
    rForward 25
  endif
  call Face(135)
  delay 1000
  r = rRange(90)
  if r<35 or r>150 then _Lost = True
  Cur = 4  
return

sub Face(dir)
  if _Lost then return
  if _Real
    rCommand(_TurnToHalfAngle,dir/2)
  else
    c=rCompass()
    if (dir>c) and (abs(dir-c)<=180)
      d=1
    else
      d=-1
    endif
    while rCompass()!=dir
      rTurn d
    wend
  endif
return

sub FindWall()
  if _Lost then return
  if _Real
    dist = 40
  else
    dist = 55
  endif
  for i= 1 to dist
    rForward 1
    s = rFeel()    
    if s&2 then rTurn -1
    if s&8 then rTurn 1
    if s&17 
      rForward 5
      break
    endif
    if s&4 then _Lost = True\break
  next
return

sub FollowWall(side)
  if side = _OnLeft
    Direction = -90
    SensorMask = 8
    REVERSE = 1
  else
    Direction = 90
    SensorMask = 2
    REVERSE = -1
  endif
  if _Real
    WallDist = 18
  else
    WallDist = 38
  endif
  while TRUE
    if _Lost then return
    r1=rRange(Direction)
    for i=1 to 10 // edit based on robot's speed
      if rFeel()&4 then _Lost=True\break;
      rForward 1
      while (rFeel()&SensorMask) // prevent obvious wall encounter
        rTurn REVERSE
      wend
      r2=rRange(Direction)      
    next
    if r2>45 then break; // EOW   
    change = r2-r1
    if change>0
      // first prioity - stay parallel with wall
      for j= 1 to 2+abs(change)
        rTurn -sign(change)*REVERSE
      next
    else
      // robot is parallel - so now try to correct dist from wall
      for k=1 to 2
      if r2>18 then rTurn -REVERSE
      if r2<18 then  rTurn REVERSE
      next
    endif
  wend
return

sub MoveThroughDoor(Direction)
  if _Lost then return
  call Face(Direction)
  while true  // rFeel must be set to feel wider than the door
    if rFeel()&4 then Lost = True \break
    if rFeel()&2 then rTurn -2
    if rFeel()&8 then rTurn 2
    if rFeel()&17 then break // doorway found
    rForward 1
  wend
  delay 1000
  if _Real
    rforward 40
  else
    rForward 50
  endif
return

DrawFloorPlan:
  LineWidth 18
  Rectangle 1,1,798,598
  LineWidth 2
  //Bedroom 1
  // Lower Wall
  Line 380,10,380,300
  LineTo 90,300
  LineTo 90,270
  LineTo 100,270
  LineTo 100,290
  LineTo 370,290
  LineTo 370,10
  //Start upper door jamb
  Line 90,190,90,170
  LineTo 10,170
  Line 10,160,90,160
  LineTo 90,140
  LineTo 100,140
  LineTo 100,190
  LineTo 90,190
  Line 90,10,90,35
  LineTo 100,35
  LineTo 100,10
  //lower hallway wall, left door jamb
  Line 10,380,20,380
  LineTo 20,390
  LineTo 10,390
  // Tub
  Line 10,530,130,530
  Line 130,595,130,390
  Circle 20,540,120,580 // tub itself
  LineTo 70,390
  LineTo 70,380
  LineTo 150,380 //right door jamb
  LineTo 150,390
  LineTo 140,390
  LineTo 140,595
  // Bedroom 2, right door jamb
  Line 230,380,500,380                          
  LineTo 500,595
  Line 490,595,490,390
  LineTo 420,390
  LineTo 420,430 // closet jamb
  LineTO 410,430
  LineTo 410,390
  LineTo 230,390
  LineTo 230,380
  Line 420,595,420,555
  LineTo 410,555
  LineTo 410,595
  // bath sink
  Line 85,390,85,460
  LineTo 130,460
  Circle 95,400,120,450
  Circle 70,475,115,510
  Rectangle 105,468,125,520
  xyText 25,425,"Bath","Times New Roman",16,,black,white
  xyText 190,125,"Bedroom 1","Times New Roman",16,,black,white
  xyText 250,475,"Bedroom 2","Times New Roman",16,,black,white
  xyText 650,400,"Kitchen","Times New Roman",16,,black,white
  xyText 525,120,"Great Room","Times New Roman",16,,black,white
  // Kitchen
  Line 500,385,560,385
  LineTo 560,530
  LineTo 730,530
  LineTo 730,445
  LineTo 790,445
  Rectangle 725,385,787,442
  Rectangle 620,540,675,575
  Rectangle 507,430,555,485 
  Call DC(519,445,8)                      
  Call DC(542,445,8)
  Call DC(519,470,8)
  Call DC(542,470,8)
  // closet doors
  Line 95,36,120,46
  LineTo 95,56
  Line 95,139,120,129
  LineTo 95,119
  Line 415,431,390,441
  LineTo 415,451
  Line 415,554,390,544
  LineTo 415,534
  
  // draw positions
  /*
  Call Pos(125,227,1)
  Call Pos(190,415,2)
  Call Pos(430,218,3)
  Call Pos(605,360,4)
  Call Pos(660,218,5)
  */
  SetColor Black,RGB(210,210,210)
  Rectangle 770,207,789,228
  Rectangle 380,207,400,228
  xyString 386,209,"B"
  xyString 776,209,"B"
  SetColor Black,White
return

sub DC(x,y,r)
  Circle x-r,y-r,x+r,y+r
return

sub Pos(x,y,n)
  c = RGB(210,210,210)
  SetColor Black,c
  Call DC(x,y,15)
  SetColor Black,white
  xyText x-4,y-10,n,"Times New Roman",16,,Black,c
return
